@@ -8,6 +8,7 @@
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2006 Paul Wise
 **	Copyright (c) 2007, 2008 Chris Moore
+**	Copyright (c) 2012-2013 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -41,6 +42,8 @@
 #include "lyr_freetype.h"
 #endif
 
+#include <pango/pangocairo.h>
+
 using namespace std;
 using namespace etl;
 using namespace synfig;
@@ -134,6 +137,11 @@ Layer_Freetype::new_font(const synfig::String &family, int style, int weight)
 		new_font_("sans serif",PANGO_STYLE_NORMAL,WEIGHT_NORMAL);
 }
 
+/*! The new_font() function try to render
+**	text until it work by simplyfing font style(s).
+** In last chance, render text as "sans serif" Normal
+** font style.
+*/
 bool
 Layer_Freetype::new_font_(const synfig::String &font_fam_, int style, int weight)
 {
@@ -911,6 +919,180 @@ Layer_Freetype::accelerated_render(Context context,Surface *surface,int quality,
 	return true;
 }
 
+////
+bool
+Layer_Freetype::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+{
+	const Point	tl(renddesc.get_tl());
+	const Point br(renddesc.get_br());
+	
+	const int	w(renddesc.get_w());
+	const int	h(renddesc.get_h());
+	
+	// Width and Height of a pixel
+	const Real pw = (br[0] - tl[0]) / w;
+	const Real ph = (br[1] - tl[1]) / h;
+	
+	const double sx(1/pw);
+	const double sy(1/ph);
+	const double tx((-tl[0]+origin[0])*sx);
+	const double ty((-tl[1]+origin[1])*sy);
+
+	if(!is_solid_color())
+	{
+		// Initially render what's behind us
+		if(!context.accelerated_cairorender(surface,quality,renddesc,cb))
+		{
+			if(cb)cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
+			return false;
+		}
+	}
+
+	// Cairo context
+	cairo_surface_t* subimage;
+	cairo_surface_t* inverted;
+	subimage=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, w, h);
+	cairo_t* subcr=cairo_create(subimage);
+	cairo_t* invertcr;
+	if(invert)
+	{
+		inverted=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, w, h);
+		invertcr=cairo_create(inverted);
+		cairo_set_source_rgba(invertcr, color.get_r(), color.get_g(), color.get_b(), color.get_a());
+		cairo_paint_with_alpha(invertcr, get_amount());
+	}
+
+	// Pango
+	PangoLayout *layout;
+	PangoFontDescription *font_description;
+	// Pango Font
+	font_description = pango_font_description_new ();
+	pango_font_description_set_family (font_description, font.c_str());
+	pango_font_description_set_weight (font_description, PangoWeight(weight));
+	pango_font_description_set_style (font_description, PangoStyle(style));
+	// The size is scaled to match Software render size (remove the scale?)
+	float sizex=1.75*fabs(size[0])*fabs(sx);
+	float sizey=1.75*fabs(size[1])*fabs(sy);
+	float vscale=sizey/sizex;
+	pango_font_description_set_absolute_size (font_description, sizex * PANGO_SCALE );
+	
+	//Pango Layout
+	layout = pango_cairo_create_layout (subcr);
+	
+	pango_layout_set_font_description (layout, font_description);
+	pango_layout_set_text (layout, text.c_str(), -1);
+	if(orient[0]<0.4)
+		pango_layout_set_alignment(layout, PANGO_ALIGN_LEFT);
+	else if(orient[0]>0.6)
+		pango_layout_set_alignment(layout, PANGO_ALIGN_RIGHT);
+	else
+		pango_layout_set_alignment(layout, PANGO_ALIGN_CENTER);
+		
+	pango_layout_set_single_paragraph_mode(layout, false);
+
+	// Calculate the logical and ink rectangles of the layout before add spacing
+	PangoRectangle ink_layout, logical_layout;
+	PangoRectangle ink_rect, logical_rect;
+	pango_layout_get_pixel_extents(layout, &ink_layout, &logical_layout);
+
+	// Spacing
+	// Horizontal
+	PangoAttrList* attrlist=pango_attr_list_new();
+	float hspace=compress>1.0?0.4*sizex*(compress-1.0):(compress<1.0)?0.5*sizex*(compress-1.0):0;
+	PangoAttribute* spacing=pango_attr_letter_spacing_new(hspace*PANGO_SCALE);
+	pango_attr_list_insert_before(attrlist, spacing);
+	pango_layout_set_attributes(layout, attrlist);
+	
+	// Vertical
+	int total_lines=pango_layout_get_line_count(layout);
+	float vspace_total=vcompress>1.0?0.4*logical_layout.height*(vcompress-1.0):(vcompress<1.0)?0.6*logical_layout.height*(vcompress-1.0):0;
+	float vspace;
+	if(total_lines>1)
+		vspace=vspace_total/(total_lines-1);
+	pango_layout_set_spacing(layout, vspace*PANGO_SCALE);
+
+	// Recalculate extents due to spacing changes
+	pango_layout_get_pixel_extents(layout, &ink_layout, &logical_layout);
+
+	// Render text
+	cairo_save(subcr);
+	cairo_set_source_rgba(subcr, color.get_r(), color.get_g(), color.get_b(), color.get_a());
+	cairo_scale(subcr, 1.0, vscale);
+	pango_cairo_update_layout(subcr, layout);
+	cairo_move_to(subcr, tx-logical_layout.width*orient[0], (ty-(logical_layout.height+vspace_total)*vscale*orient[1])/vscale);
+	pango_cairo_show_layout(subcr, layout);
+
+	// Debug ink and logical lines
+	if(0)
+	{
+		pango_layout_get_pixel_extents(layout, &ink_rect, &logical_rect);	
+		// Render logical and ink rectangles
+		cairo_save(subcr);
+		cairo_set_source_rgb(subcr, 0.0, 1.0, 0.0);
+		cairo_set_line_width(subcr, 1.0);
+		cairo_rectangle(subcr, tx+ink_rect.x-0.5-logical_layout.width*orient[0],
+							ty+ink_rect.y-0.5-(logical_layout.height+vspace_total)*orient[1],
+							ink_rect.width,
+							ink_rect.height);
+		cairo_stroke(subcr);
+		cairo_restore(subcr);
+
+		cairo_save(subcr);
+		cairo_set_line_width(subcr, 1.0);
+		cairo_set_source_rgb(subcr, 0.0, 0.0, 1.0);
+		cairo_rectangle(subcr, tx+logical_rect.x-0.5-logical_layout.width*orient[0],
+							ty+logical_rect.y-0.5-(logical_layout.height+vspace_total)*orient[1],
+							logical_rect.width,
+							logical_rect.height);
+		cairo_stroke(subcr);
+		cairo_move_to(subcr, tx+2, ty);
+		cairo_arc(subcr, tx, ty, 2.0, 0, 2*3.141516);
+		cairo_fill(subcr);
+		cairo_restore(subcr);
+	}
+	cairo_restore(subcr);
+	
+	// Render the text on the target surface with the proper operator
+	if(invert)
+	{
+		cairo_set_source_surface(invertcr, subimage, 0,0);
+		cairo_set_operator(invertcr, CAIRO_OPERATOR_DEST_OUT);
+		cairo_paint_with_alpha(invertcr, get_amount());
+	}
+	cairo_t* cr=cairo_create(surface);
+	cairo_save(cr);
+	if(invert)
+		cairo_set_source_surface(cr, inverted, 0, 0);
+	else
+		cairo_set_source_surface(cr, subimage, 0, 0);
+	if(is_solid_color())
+	{
+			cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+			cairo_paint(cr);
+	}
+	else
+	{
+		cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
+	}
+	cairo_restore(cr);
+
+	// Destroy and return
+	cairo_surface_destroy(subimage);
+	cairo_destroy(subcr);
+	if(invert)
+	{
+		cairo_surface_destroy(inverted);
+		cairo_destroy(invertcr);
+	}
+	cairo_destroy(cr);
+	pango_attr_list_unref(attrlist);
+	g_object_unref (layout);
+	pango_font_description_free (font_description);
+	return true;
+}
+////
+
+
 synfig::Rect
 Layer_Freetype::get_bounding_rect()const
 {
