@@ -7,7 +7,7 @@
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2007, 2008 Chris Moore
-**	Copyright (c) 2011 Carlos López
+**	Copyright (c) 2011-2013 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -271,3 +271,9 @@ Layer_TimeLoop::accelerated_render(Context context,Surface *surface,int quality,
 {
 	return context.accelerated_render(surface,quality,renddesc,cb);
 }
+
+bool
+Layer_TimeLoop::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+{
+	return context.accelerated_cairorender(surface,quality,renddesc,cb);
+}
