@@ -6,7 +6,7 @@
 **
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
-**	Copyright (c) 2011 Carlos López
+**	Copyright (c) 2011-2013 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -176,6 +176,13 @@ Twirl::get_color(Context context, const Point &pos)const
 	return context.get_color(distort(pos));
 }
 
+CairoColor
+Twirl::get_cairocolor(Context context, const Point &pos)const
+{
+	return context.get_cairocolor(distort(pos));
+}
+
+
 class Twirl_Trans : public Transform
 {
 	etl::handle<const Twirl> layer;
