@@ -6,6 +6,7 @@
 **
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
+**	Copyright (c) 2012-2013 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -157,6 +158,25 @@ Zoom::accelerated_render(Context context,Surface *surface,int quality, const Ren
 	return context.accelerated_render(surface,quality,desc,cb);
 }
 
+/////
+bool
+Zoom::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+{
+	Vector::value_type zoomfactor=1.0/exp(amount);
+	RendDesc desc(renddesc);
+	desc.clear_flags();
+	
+    // Adjust the top_left and bottom_right points
+	// for our zoom amount
+	desc.set_tl((desc.get_tl()-center)*zoomfactor+center);
+	desc.set_br((desc.get_br()-center)*zoomfactor+center);
+	
+	// Render the scene
+	return context.accelerated_cairorender(surface,quality,desc,cb);
+}
+/////
+
+
 synfig::Rect
 Zoom::get_full_bounding_rect(synfig::Context context)const
 {
