@@ -7,6 +7,7 @@
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2008 Chris Moore
+**	Copyright (c) 2012-2013 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -37,7 +38,7 @@ using namespace synfig;
 using namespace std;
 using namespace etl;
 
-class Blur_Layer : public synfig::Layer_Composite
+class Blur_Layer : public Layer_Composite
 {
 	SYNFIG_LAYER_MODULE_EXT
 private:
@@ -47,13 +48,14 @@ private:
 public:
 	Blur_Layer();
 
-	virtual bool set_param(const String & param, const synfig::ValueBase &value);
+	virtual bool set_param(const String & param, const ValueBase &value);
 
 	virtual ValueBase get_param(const String & param)const;
 
 	virtual Color get_color(Context context, const Point &pos)const;
 
 	virtual bool accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
+	virtual bool accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
 	virtual synfig::Rect get_full_bounding_rect(Context context)const;
 
 	virtual Vocab get_param_vocab()const;
