@@ -7,6 +7,7 @@
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2008 Chris Moore
+**	Copyright (c) 2012-2013 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -40,31 +41,35 @@
 /* === T Y P E D E F S ===================================================== */
 
 /* === C L A S S E S & S T R U C T S ======================================= */
+using namespace synfig;
+using namespace std;
+using namespace etl;
 
-class Halftone2 : public synfig::Layer_Composite
+class Halftone2 : public Layer_Composite
 {
 	SYNFIG_LAYER_MODULE_EXT
 
 private:
 
 	Halftone halftone;
-	synfig::Color	color_dark;
-	synfig::Color	color_light;
+	Color	color_dark;
+	Color	color_light;
 
-	synfig::Color color_func(const synfig::Point &x, float supersample,const synfig::Color &under_color)const;
+	Color color_func(const Point &x, float supersample,const Color &under_color)const;
 
-	float calc_supersample(const synfig::Point &x, float pw,float ph)const;
+	float calc_supersample(const Point &x, float pw,float ph)const;
 
-	//float halftone_func(synfig::Point x)const;
+	//float halftone_func(Point x)const;
 
 public:
 	Halftone2();
 
-	virtual bool set_param(const synfig::String &param, const synfig::ValueBase &value);
-	virtual synfig::ValueBase get_param(const synfig::String &param)const;
-	virtual synfig::Color get_color(synfig::Context context, const synfig::Point &pos)const;
-	virtual bool accelerated_render(synfig::Context context,synfig::Surface *surface,int quality, const synfig::RendDesc &renddesc, synfig::ProgressCallback *cb)const;
-	synfig::Layer::Handle hit_check(synfig::Context context, const synfig::Point &point)const;
+	virtual bool set_param(const String &param, const ValueBase &value);
+	virtual ValueBase get_param(const String &param)const;
+	virtual Color get_color(Context context, const Point &pos)const;
+	virtual bool accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
+	virtual bool accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
+	Layer::Handle hit_check(Context context, const Point &point)const;
 	virtual Vocab get_param_vocab()const;
 	virtual bool reads_context()const { return true; }
 }; // END of class Halftone2
