@@ -7,6 +7,7 @@
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2007-2008 Chris Moore
+**	Copyright (c) 2012-2013 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -409,3 +410,72 @@ Halftone3::accelerated_render(Context context,Surface *surface,int quality, cons
 
 	return true;
 }
+
+////
+bool
+Halftone3::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+{
+	SuperCallback supercb(cb,0,9500,10000);
+	
+	if(!context.accelerated_cairorender(surface,quality,renddesc,&supercb))
+		return false;
+	if(get_amount()==0)
+		return true;
+	
+	CairoSurface csurface(surface);
+	if(!csurface.map_cairo_image())
+		return false;
+	const Real pw(renddesc.get_pw()),ph(renddesc.get_ph());
+	const Point tl(renddesc.get_tl());
+	const int w(csurface.get_w());
+	const int h(csurface.get_h());
+	const float supersample_size(abs(pw/(tone[0].size).mag()));
+	
+	CairoSurface::pen pen(csurface.begin());
+	Point pos;
+	int x,y;
+	
+	if(is_solid_color())
+	{
+		for(y=0,pos[1]=tl[1];y<h;y++,pen.inc_y(),pen.dec_x(x),pos[1]+=ph)
+			for(x=0,pos[0]=tl[0];x<w;x++,pen.inc_x(),pos[0]+=pw)
+				pen.put_value(
+					CairoColor(color_func(
+							pos,
+							supersample_size,
+							Color(pen.get_value().demult_alpha())
+							)).premult_alpha()
+				);
+	}
+	else
+	{
+		for(y=0,pos[1]=tl[1];y<h;y++,pen.inc_y(),pen.dec_x(x),pos[1]+=ph)
+			for(x=0,pos[0]=tl[0];x<w;x++,pen.inc_x(),pos[0]+=pw)
+			{
+				Color val=Color(pen.get_value().demult_alpha());
+				pen.put_value(
+					  CairoColor(Color::blend(
+								  color_func(
+											 pos,
+											 supersample_size,
+											 val
+											 ),
+								  val,
+								  get_amount(),
+								  get_blend_method()
+								  ).clamped()
+						 ).premult_alpha()
+				);
+			}
+		
+	}
+	
+	csurface.unmap_cairo_image();
+	// Mark our progress as finished
+	if(cb && !cb->amount_complete(10000,10000))
+		return false;
+	
+	return true;
+}
+
+////
