@@ -6,7 +6,7 @@
 **
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
-**	Copyright (c) 2011 Carlos López
+**	Copyright (c) 2011-2013 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -130,7 +130,7 @@ Advanced_Outline::sync()
 	clear();
 	if (!bline_.get_list().size())
 	{
-		synfig::warning(string("Advanced_Outline::sync():")+N_("No vertices in bline " + string("\"") + get_description() + string("\"")));
+		synfig::warning(string("Advanced_Outline::sync():")+N_("No vertices in spline " + string("\"") + get_description() + string("\"")));
 		return;
 	}
 	try
@@ -1122,7 +1122,7 @@ Advanced_Outline::get_param_vocab()const
 	ret.push_back(ParamDesc("bline")
 		.set_local_name(_("Vertices"))
 		.set_origin("origin")
-		.set_description(_("A list of BLine Points"))
+		.set_description(_("A list of spline points"))
 	);
 	ret.push_back(ParamDesc("width")
 		.set_is_distance()
@@ -1136,7 +1136,7 @@ Advanced_Outline::get_param_vocab()const
 	);
 	ret.push_back(ParamDesc(ValueBase(),"start_tip")
 		.set_local_name(_("Tip Type at Start"))
-		.set_description(_("Defines the Tip type of the first bline point when bline is unlooped"))
+		.set_description(_("Defines the Tip type of the first spline point when spline is unlooped"))
 		.set_hint("enum")
 		.add_enum_value(WidthPoint::TYPE_ROUNDED,"rounded", _("Rounded Stop"))
 		.add_enum_value(WidthPoint::TYPE_SQUARED,"squared", _("Squared Stop"))
@@ -1145,7 +1145,7 @@ Advanced_Outline::get_param_vocab()const
 		);
 	ret.push_back(ParamDesc(ValueBase(),"end_tip")
 		.set_local_name(_("Tip Type at End"))
-		.set_description(_("Defines the Tip type of the last bline point when bline is unlooped"))
+		.set_description(_("Defines the Tip type of the last spline point when spline is unlooped"))
 		.set_hint("enum")
 		.add_enum_value(WidthPoint::TYPE_ROUNDED,"rounded", _("Rounded Stop"))
 		.add_enum_value(WidthPoint::TYPE_SQUARED,"squared", _("Squared Stop"))
@@ -1166,7 +1166,7 @@ Advanced_Outline::get_param_vocab()const
 	);
 	ret.push_back(ParamDesc("homogeneous")
 		.set_local_name(_("Homogeneous"))
-		.set_description(_("When true, widthpoints positions are bline length based"))
+		.set_description(_("When true, widthpoints positions are spline length based"))
 	);
 	ret.push_back(ParamDesc("wplist")
 		.set_local_name(_("Width Point List"))
@@ -1244,6 +1244,8 @@ Advanced_Outline::connect_bline_to_wplist(etl::loose_handle<ValueNode> x)
 {
 	if(x->get_type() != ValueBase::TYPE_LIST)
 		return false;
+	if((*x)(Time(0)).empty())
+		return false;
 	if((*x)(Time(0)).get_list().front().get_type() != ValueBase::TYPE_BLINEPOINT)
 		return false;
 	ValueNode::LooseHandle vnode;
@@ -1262,6 +1264,8 @@ Advanced_Outline::connect_bline_to_dilist(etl::loose_handle<ValueNode> x)
 {
 	if(x->get_type() != ValueBase::TYPE_LIST)
 		return false;
+	if((*x)(Time(0)).empty())
+		return false;
 	if((*x)(Time(0)).get_list().front().get_type() != ValueBase::TYPE_BLINEPOINT)
 		return false;
 	ValueNode::LooseHandle vnode;
@@ -1494,3 +1498,183 @@ Advanced_Outline::add_cusp(std::vector<Point> &side_a, std::vector<Point> &side_
 		break;
 	}
 }
+
+bool
+Advanced_Outline::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+{
+	// Grab the rgba values
+	const float r(color.get_r());
+	const float g(color.get_g());
+	const float b(color.get_b());
+	const float a(color.get_a());
+	
+	// Window Boundaries
+	const Point	tl(renddesc.get_tl());
+	const Point br(renddesc.get_br());
+	const int	w(renddesc.get_w());
+	const int	h(renddesc.get_h());
+	
+	// Width and Height of a pixel
+	const Real pw = (br[0] - tl[0]) / w;
+	const Real ph = (br[1] - tl[1]) / h;
+	
+	// These are the scale and translation values
+	const double sx(1/pw);
+	const double sy(1/ph);
+	const double tx((-tl[0]+origin[0])*sx);
+	const double ty((-tl[1]+origin[1])*sy);
+
+	cairo_t* cr=cairo_create(surface);
+
+	// Let's render the outline in other surface
+	// Initially I'll fill it completely with the alpha color
+	cairo_surface_t* subimage;
+	// Let's calculate the subimage dimensions based on the feather value
+	//so make a separate surface
+	RendDesc	workdesc(renddesc);
+	int halfsizex(0), halfsizey(0);
+	if(feather && quality != 10)
+	{
+		//the expanded size = 1/2 the size in each direction rounded up
+		halfsizex = (int) (abs(feather*.5/pw) + 3),
+		halfsizey = (int) (abs(feather*.5/ph) + 3);
+		
+		//expand by 1/2 size in each direction on either side
+		switch(blurtype)
+		{
+			case Blur::DISC:
+			case Blur::BOX:
+			case Blur::CROSS:
+			{
+				workdesc.set_subwindow(-max(1,halfsizex),-max(1,halfsizey),w+2*max(1,halfsizex),h+2*max(1,halfsizey));
+				break;
+			}
+			case Blur::FASTGAUSSIAN:
+			{
+				if(quality < 4)
+				{
+					halfsizex*=2;
+					halfsizey*=2;
+				}
+				workdesc.set_subwindow(-max(1,halfsizex),-max(1,halfsizey),w+2*max(1,halfsizex),h+2*max(1,halfsizey));
+				break;
+			}
+			case Blur::GAUSSIAN:
+			{
+#define GAUSSIAN_ADJUSTMENT		(0.05)
+				Real	pw = (Real)workdesc.get_w()/(workdesc.get_br()[0]-workdesc.get_tl()[0]);
+				Real 	ph = (Real)workdesc.get_h()/(workdesc.get_br()[1]-workdesc.get_tl()[1]);
+				
+				pw=pw*pw;
+				ph=ph*ph;
+				
+				halfsizex = (int)(abs(pw)*feather*GAUSSIAN_ADJUSTMENT+0.5);
+				halfsizey = (int)(abs(ph)*feather*GAUSSIAN_ADJUSTMENT+0.5);
+				
+				halfsizex = (halfsizex + 1)/2;
+				halfsizey = (halfsizey + 1)/2;
+				workdesc.set_subwindow( -halfsizex, -halfsizey, w+2*halfsizex, h+2*halfsizey );
+				break;
+#undef GAUSSIAN_ADJUSTMENT
+			}
+		}
+	}
+	subimage=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, workdesc.get_w(), workdesc.get_h());
+	cairo_t* subcr=cairo_create(subimage);
+
+	cairo_save(subcr);
+	cairo_set_source_rgba(subcr, r, g, b, a);
+	// Now let's check if it is inverted
+	if(invert)
+	{
+		cairo_paint(subcr);
+	}
+	// Draw the outline
+	// Calculate new translations after expand the tile
+	const double extx((-workdesc.get_tl()[0]+origin[0])*sx);
+	const double exty((-workdesc.get_tl()[1]+origin[1])*sy);
+	
+	cairo_translate(subcr, extx , exty);
+	cairo_scale(subcr, sx, sy);
+	switch(winding_style)
+	{
+		case WINDING_NON_ZERO:
+			cairo_set_fill_rule(subcr, CAIRO_FILL_RULE_WINDING);
+			break;
+		default:
+			cairo_set_fill_rule(subcr, CAIRO_FILL_RULE_EVEN_ODD);
+			break;
+	}
+	if(!antialias)
+		cairo_set_antialias(subcr, CAIRO_ANTIALIAS_NONE);
+	if(invert)
+		cairo_set_operator(subcr, CAIRO_OPERATOR_CLEAR);
+	else
+		cairo_set_operator(subcr, CAIRO_OPERATOR_OVER);
+
+	// For any quality...
+	Layer_Shape::shape_to_cairo(subcr);
+	cairo_clip(subcr);
+	cairo_paint(subcr);
+	
+	cairo_restore(subcr);
+
+	if(feather && quality!=10)
+	{
+		etl::surface<float>	shapesurface;
+		shapesurface.set_wh(workdesc.get_w(),workdesc.get_h());
+		shapesurface.clear();
+		
+		CairoSurface cairosubimage(subimage);
+		if(!cairosubimage.map_cairo_image())
+		{
+			synfig::info("map cairo image failed");
+			return false;
+		}
+		// Extract the alpha values:
+		int x, y;
+		int wh(workdesc.get_h()), ww(workdesc.get_w());
+		float div=1.0/((float)(CairoColor::ceil));
+		for(y=0; y<wh; y++)
+			for(x=0;x<ww;x++)
+				shapesurface[y][x]=cairosubimage[y][x].get_a()*div;
+		// Blue the alpha values
+		Blur(feather,feather,blurtype,cb)(shapesurface,workdesc.get_br()-workdesc.get_tl(),shapesurface);
+		// repaint the cairosubimage with the result
+		Color ccolor(color);
+		for(y=0; y<wh; y++)
+			for(x=0;x<ww;x++)
+			{
+				float a=shapesurface[y][x];
+				ccolor.set_a(a);
+				ccolor.clamped();
+				cairosubimage[y][x]=CairoColor(ccolor).premult_alpha();
+			}
+		
+		cairosubimage.unmap_cairo_image();
+	}
+	// Put the (feathered) outline on the surface
+	if(!is_solid_color()) // we need to render the context before
+		if(!context.accelerated_cairorender(surface,quality,renddesc,cb))
+		{
+			if(cb)
+				cb->error(strprintf(__FILE__"%d: Accelerated Cairo Renderer Failure",__LINE__));
+			cairo_destroy(cr);
+			cairo_destroy(subcr);
+			cairo_surface_destroy(subimage);
+			return false;
+		}
+	cairo_save(cr);
+	const double px(tl[0]-workdesc.get_tl()[0]);
+	const double py(tl[1]-workdesc.get_tl()[1]);
+	cairo_set_source_surface(cr, subimage, -px*sx, -py*sy);
+	cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
+	cairo_restore(cr);
+	cairo_surface_destroy(subimage);
+	cairo_destroy(subcr);
+	cairo_destroy(cr);
+	
+	return true;
+
+}
+
