@@ -7,7 +7,7 @@
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2007, 2008 Chris Moore
-**	Copyright (c) 2011 Carlos López
+**	Copyright (c) 2011-2013 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -200,3 +200,101 @@ CheckerBoard::accelerated_render(Context context,Surface *surface,int quality, c
 
 	return true;
 }
+
+//////////
+
+bool
+CheckerBoard::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+{
+	SuperCallback supercb(cb,0,9500,10000);
+
+	if(!is_solid_color())
+		if(!context.accelerated_cairorender(surface,quality,renddesc,&supercb))
+			return false;
+	
+	if(get_amount()==0)
+		return true;
+
+	const float r(color.get_r());
+	const float g(color.get_g());
+	const float b(color.get_b());
+	const float a(color.get_a());
+	
+	const Point	tl(renddesc.get_tl());
+	const Point br(renddesc.get_br());
+	
+	const int	w(renddesc.get_w());
+	const int	h(renddesc.get_h());
+	
+	// Width and Height of a pixel
+	const Real pw = (br[0] - tl[0]) / w;
+	const Real ph = (br[1] - tl[1]) / h;
+
+	// These are translation and scale values
+	const double tx(-tl[0]/pw);
+	const double ty(-tl[1]/ph);
+	const double sx(1/pw);
+	const double sy(1/ph);
+	
+	Point newsize(size);
+	// Normalize the size
+	if(newsize[0] <0.0) newsize[0]=-newsize[0];
+	if(newsize[1] <0.0) newsize[1]=-newsize[1];
+	// Calculate one average size that fits in a number integer of pixels
+	newsize[0]=((int)(2.0*newsize[0]*sx))/(2.0*sx);
+	newsize[1]=((int)(2.0*newsize[1]*sy))/(2.0*sy);
+	
+	// Now let's render the minimum checkerboard in other surface
+	// Initially I'll fill it completely with the alpha color
+	// Create a similar image with the same dimensions than the minimum checkerboard
+	
+	RendDesc desc(renddesc);
+	// this will modify the w and h values in pixels.
+	desc.set_flags(RendDesc::PX_ASPECT|RendDesc::IM_SPAN);
+	desc.set_tl_br(Point(-newsize[0], +newsize[1]), Point(+newsize[0], -newsize[1]));
+	cairo_surface_t* subimage=cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, desc.get_w(), desc.get_h());
+	cairo_t* subcr=cairo_create(subimage);
+	cairo_save(subcr);
+	cairo_set_source_rgba(subcr, r, g, b, a);
+	cairo_rectangle(subcr, 0, 0, desc.get_w()/2, desc.get_h()/2);
+	cairo_clip(subcr);
+	cairo_paint(subcr);
+	cairo_restore(subcr);
+	cairo_save(subcr);
+	cairo_set_source_rgba(subcr, r, g, b, a);
+	cairo_rectangle(subcr, desc.get_w()/2, desc.get_h()/2, desc.get_w()/2, desc.get_h()/2);
+	cairo_clip(subcr);
+	cairo_paint(subcr);
+	cairo_restore(subcr);
+	
+	cairo_t* cr=cairo_create(surface);
+	cairo_save(cr);
+	
+	cairo_translate(cr, tx+origin[0]*sx -newsize[0]*sx, ty+origin[1]*sy+newsize[1]*sy);
+	cairo_pattern_t* pattern=cairo_pattern_create_for_surface(subimage);
+	cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
+	cairo_set_source(cr, pattern);
+	if(is_solid_color())
+	{
+		cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+		cairo_paint_with_alpha(cr, get_amount());
+	}
+	else
+	{
+		cairo_paint_with_alpha_operator(cr, get_amount(), get_blend_method());
+	}
+	cairo_restore(cr);
+	cairo_surface_destroy(subimage);
+	cairo_pattern_destroy(pattern);
+	cairo_destroy(subcr);
+	cairo_destroy(cr);
+
+	// Mark our progress as finished
+	if(cb && !cb->amount_complete(10000,10000))
+		return false;
+	
+	return true;
+}
+
+//////////
+
