@@ -546,7 +546,7 @@ CurveGradient::get_param_vocab()const
 				  .set_local_name(_("Vertices"))
 				  .set_origin("origin")
 				  .set_hint("width")
-				  .set_description(_("A list of BLine Points"))
+				  .set_description(_("A list of spline points"))
 	);
 	ret.push_back(ParamDesc("gradient")
 				  .set_local_name(_("Gradient"))
