@@ -7,6 +7,7 @@
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2007, 2008 Chris Moore
+**	Copyright (c) 2012-2013 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -60,6 +61,8 @@ private:
 	mutable synfig::Time curr_time;
 
 	synfig::Color color_func(const synfig::Point &x, float supersample,synfig::Context context)const;
+	synfig::CairoColor cairocolor_func(const synfig::Point &x, float supersample,synfig::Context context)const;
+	synfig::Point point_func(const synfig::Point &point)const;
 
 	float calc_supersample(const synfig::Point &x, float pw,float ph)const;
 
@@ -69,6 +72,7 @@ public:
 	virtual bool set_param(const synfig::String &param, const synfig::ValueBase &value);
 	virtual synfig::ValueBase get_param(const synfig::String &param)const;
 	virtual synfig::Color get_color(synfig::Context context, const synfig::Point &pos)const;
+	virtual synfig::CairoColor get_cairocolor(synfig::Context context, const synfig::Point &pos)const;
 	//virtual bool accelerated_render(synfig::Context context,synfig::Surface *surface,int quality, const synfig::RendDesc &renddesc, synfig::ProgressCallback *cb)const;
 	synfig::Layer::Handle hit_check(synfig::Context context, const synfig::Point &point)const;
 	virtual void set_time(synfig::Context context, synfig::Time time)const;
