@@ -7,7 +7,7 @@
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2007 Chris Moore
-**	Copyright (c) 2011 Carlos López
+**	Copyright (c) 2011-2013 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -312,6 +312,17 @@ Noise::get_color(Context context, const Point &point)const
 		return Color::blend(color,context.get_color(point),get_amount(),get_blend_method());
 }
 
+CairoColor
+Noise::get_cairocolor(Context context, const Point &point)const
+{
+	const CairoColor color(color_func(point,0,context));
+	
+	if(get_amount()==1.0 && get_blend_method()==Color::BLEND_STRAIGHT)
+		return color;
+	else
+		return CairoColor::blend(color,context.get_cairocolor(point),get_amount(),get_blend_method());
+}
+
 bool
 Noise::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
