@@ -389,7 +389,7 @@ Plant::get_param_vocab()const
 
 	ret.push_back(ParamDesc("bline")
 		.set_local_name(_("Vertices"))
-		.set_description(_("A list of BLine Points"))
+		.set_description(_("A list of spline points"))
 		.set_origin("origin")
 		.set_hint("width")
 	);
@@ -417,12 +417,12 @@ Plant::get_param_vocab()const
 
 	ret.push_back(ParamDesc("velocity")
 		.set_local_name(_("Tangential Velocity"))
-		.set_description(_("Amount to which shoots tend to grow along the tangent to the BLine"))
+		.set_description(_("Amount to which shoots tend to grow along the tangent to the spline"))
 	);
 
 	ret.push_back(ParamDesc("perp_velocity")
 		.set_local_name(_("Perpendicular Velocity"))
-		.set_description(_("Amount to which shoots tend to grow perpendicular to the tangent to the BLine"))
+		.set_description(_("Amount to which shoots tend to grow perpendicular to the tangent to the spline"))
 	);
 
 	ret.push_back(ParamDesc("size")
@@ -458,7 +458,7 @@ Plant::get_param_vocab()const
 
 	ret.push_back(ParamDesc("sprouts")
 		.set_local_name(_("Sprouts"))
-		.set_description(_("Number of places that growth occurs on each bline section"))
+		.set_description(_("Number of places that growth occurs on each spline section"))
 	);
 
 	ret.push_back(ParamDesc("random_factor")
@@ -473,7 +473,7 @@ Plant::get_param_vocab()const
 
 	ret.push_back(ParamDesc("use_width")
 		.set_local_name(_("Use Width"))
-		.set_description(_("Scale the velocity by the bline's width"))
+		.set_description(_("Scale the velocity by the spline's width"))
 	);
 
 	return ret;
