@@ -7,6 +7,7 @@
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2008 Chris Moore
+**	Copyright (c) 2012-2013 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -32,12 +33,9 @@
 
 #include "context.h"
 #include "layer.h"
-#include "layer_composite.h"
 #include "string.h"
 #include "vector.h"
 #include "color.h"
-#include "surface.h"
-#include "renddesc.h"
 #include "valuenode.h"
 
 #endif
@@ -111,6 +109,33 @@ Context::get_color(const Point &pos)const
 	return (*context)->get_color(context+1, pos);
 }
 
+CairoColor
+Context::get_cairocolor(const Point &pos)const
+{
+	Context context(*this);
+	
+	while(!context->empty())
+	{
+		// If this layer is active, then go
+		// ahead and break out of the loop
+		if((*context)->active())
+			break;
+		
+		// Otherwise, we want to keep searching
+		// till we find either an active layer,
+		// or the end of the layer list
+		++context;
+	}
+	
+	// If this layer isn't defined, return alpha
+	if((context)->empty()) return CairoColor::alpha();
+	
+	RWLock::ReaderLock lock((*context)->get_rw_lock());
+	
+	return (*context)->get_cairocolor(context+1, pos);
+}
+
+
 Rect
 Context::get_full_bounding_rect()const
 {
@@ -163,156 +188,6 @@ Context::get_full_bounding_rect()const
 	each layer can do work before or after the other work is done... so both values must be recorded...
 */
 
-bool
-Context::accelerated_render(Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb) const
-{
-#ifdef SYNFIG_PROFILE_LAYERS
-	String layer_name(curr_layer);
-
-	//sum the pre-work done by layer above us... (curr_layer is layer above us...)
-	if(depth>0)
-	{
-		time_table[curr_layer]+=profile_timer();
-		//if(run_table.count(curr_layer))run_table[curr_layer]++;
-		//	else run_table[curr_layer]=1;
-	}
-#endif	// SYNFIG_PROFILE_LAYERS
-
-	const Rect bbox(renddesc.get_rect());
-
-	// this is going to be set to true if this layer contributes
-	// nothing, but it's a straight blend with non-zero amount, and so
-	// it has an effect anyway
-	bool straight_and_empty = false;
-	etl::handle<Layer_Composite> composite;
-	Context context(*this);
-
-	for(;!(context)->empty();++context)
-	{
-		// If we are not active then move on to next layer
-		if(!(*context)->active())
-			continue;
-
-		const Rect layer_bounds((*context)->get_bounding_rect());
-		composite = etl::handle<Layer_Composite>::cast_dynamic(*context);
-
-		// If the box area is less than zero or the boxes do not
-		// intersect then move on to next layer, unless the layer is
-		// using a straight blend and has a non-zero amount, in which
-		// case it will still affect the result
-		if(layer_bounds.area() <= 0.0000000000001 || !(layer_bounds && bbox))
-		{
-			if (composite &&
-				Color::is_straight(composite->get_blend_method()) &&
-				composite->get_amount() != 0.0f)
-			{
-				straight_and_empty = true;
-				break;
-			}
-			continue;
-		}
-
-		// If this layer has Straight as the blend method and amount
-		// is 1.0, and the layer doesn't depend on its context, then
-		// we don't want to render the context
-		if (composite &&
-			composite->get_blend_method() == Color::BLEND_STRAIGHT &&
-			composite->get_amount() == 1.0f &&
-			!composite->reads_context())
-		{
-			Layer::Handle layer = *context;
-			while (!context->empty()) context++; // skip the context
-			return layer->accelerated_render(context,surface,quality,renddesc, cb);
-		}
-
-		// Break out of the loop--we have found a good layer
-		break;
-	}
-
-	// If this layer isn't defined, return alpha
-	if (context->empty() || (straight_and_empty && composite->get_amount() == 1.0f))
-	{
-#ifdef SYNFIG_DEBUG_LAYERS
-		synfig::info("Context::accelerated_render(): Hit end of list");
-#endif	// SYNFIG_DEBUG_LAYERS
-		surface->set_wh(renddesc.get_w(),renddesc.get_h());
-		surface->clear();
-#ifdef SYNFIG_PROFILE_LAYERS
-		profile_timer.reset();
-#endif	// SYNFIG_PROFILE_LAYERS
-		return true;
-	}
-
-#ifdef SYNFIG_DEBUG_LAYERS
-	synfig::info("Context::accelerated_render(): Descending into %s",(*context)->get_name().c_str());
-#endif	// SYNFIG_DEBUG_LAYERS
-
-	try {
-		RWLock::ReaderLock lock((*context)->get_rw_lock());
-
-#ifdef SYNFIG_PROFILE_LAYERS
-	//go down one layer :P
-	depth++;
-	curr_layer=(*context)->get_name();	//make sure the layer inside is referring to the correct layer outside
-	profile_timer.reset(); 										// +
-#endif	// SYNFIG_PROFILE_LAYERS
-
-	bool ret;
-
-	// this layer doesn't draw anything onto the canvas we're
-	// rendering, but it uses straight blending, so we need to render
-	// the stuff under us and then blit transparent pixels over it
-	// using the appropriate 'amount'
-	if (straight_and_empty)
-	{
-		if ((ret = Context((context+1)).accelerated_render(surface,quality,renddesc,cb)))
-		{
-			Surface clearsurface;
-			clearsurface.set_wh(renddesc.get_w(),renddesc.get_h());
-			clearsurface.clear();
-
-			Surface::alpha_pen apen(surface->begin());
-			apen.set_alpha(composite->get_amount());
-			apen.set_blend_method(composite->get_blend_method());
-
-			clearsurface.blit_to(apen);
-		}
-	}
-	else
-		ret = (*context)->accelerated_render(context+1,surface,quality,renddesc, cb);
-
-#ifdef SYNFIG_PROFILE_LAYERS
-	//post work for the previous layer
-	time_table[curr_layer]+=profile_timer();							//-
-	if(run_table.count(curr_layer))run_table[curr_layer]++;
-		else run_table[curr_layer]=1;
-
-	depth--;
-	curr_layer = layer_name; //we are now onto this layer (make sure the post gets recorded correctly...
-
-	//print out the table it we're done...
-	if(depth==0) _print_profile_report(),time_table.clear(),run_table.clear();
-	profile_timer.reset();												//+
-#endif	// SYNFIG_PROFILE_LAYERS
-
-	return ret;
-	}
-	catch(std::bad_alloc)
-	{
-		synfig::error("Context::accelerated_render(): Layer \"%s\" threw a bad_alloc exception!",(*context)->get_name().c_str());
-#ifdef _DEBUG
-		return false;
-#else  // _DEBUG
-		++context;
-		return context.accelerated_render(surface, quality, renddesc, cb);
-#endif	// _DEBUG
-	}
-	catch(...)
-	{
-		synfig::error("Context::accelerated_render(): Layer \"%s\" threw an exception, rethrowing...",(*context)->get_name().c_str());
-		throw;
-	}
-}
 
 void
 Context::set_time(Time time)const
@@ -384,7 +259,15 @@ Context::set_dirty_outlines()
 	}
 }
 
+void
+Context::set_render_method(RenderMethod x)
+{
+	Context context(*this);
 
+	if((context)->empty()) return;
+	
+	(*context)->set_render_method(context+1, x);
+}
 void
 Context::set_time(Time time,const Vector &/*pos*/)const
 {
@@ -445,3 +328,282 @@ Context::hit_check(const Point &pos)const
 
 	return (*context)->hit_check(context+1, pos);
 }
+
+
+bool
+Context::accelerated_render(Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb) const
+{
+#ifdef SYNFIG_PROFILE_LAYERS
+	String layer_name(curr_layer);
+	//sum the pre-work done by layer above us... (curr_layer is layer above us...)
+	if(depth>0)
+	{
+		time_table[curr_layer]+=profile_timer();
+		//if(run_table.count(curr_layer))run_table[curr_layer]++;
+		//	else run_table[curr_layer]=1;
+	}
+#endif	// SYNFIG_PROFILE_LAYERS
+	
+	const Rect bbox(renddesc.get_rect());
+	// this is going to be set to true if this layer contributes
+	// nothing, but it's a straight blend with non-zero amount, and so
+	// it has an effect anyway
+	bool straight_and_empty = false;
+	etl::handle<Layer_Composite> composite;
+	Context context(*this);
+	// Run all layers until context is empty
+	for(;!(context)->empty();++context)
+	{
+		// If we are not active then move on to next layer
+		if(!(*context)->active())
+			continue;
+		const Rect layer_bounds((*context)->get_bounding_rect());
+		// Cast current layer to composite
+		composite = etl::handle<Layer_Composite>::cast_dynamic(*context);
+		// If the box area is less than zero or the boxes do not
+		// intersect then move on to next layer, unless the layer is
+		// using a straight blend and has a non-zero amount, in which
+		// case it will still affect the result
+		if(layer_bounds.area() <= 0.0000000000001 || !(layer_bounds && bbox))
+		{
+			if (composite &&
+				Surface::value_type::is_straight(composite->get_blend_method()) &&
+				composite->get_amount() != 0.0f)
+			{
+				straight_and_empty = true;
+				break;
+			}
+			continue;
+		}
+		// If this layer has Straight as the blend method and amount
+		// is 1.0, and the layer doesn't depend on its context, then
+		// we don't want to render the context
+		if (composite &&
+			composite->get_blend_method() == Color::BLEND_STRAIGHT &&
+			composite->get_amount() == 1.0f &&
+			!composite->reads_context())
+		{
+			Layer::Handle layer = *context;
+			while (!context->empty()) context++; // skip the context
+			return layer->accelerated_render(context,surface,quality,renddesc, cb);
+		}
+		// Break out of the loop--we have found a good layer
+		break;
+	}
+	// If this layer isn't defined, return alpha
+	if (context->empty() || (straight_and_empty && composite->get_amount() == 1.0f))
+	{
+#ifdef SYNFIG_DEBUG_LAYERS
+		synfig::info("Context::accelerated_render(): Hit end of list");
+#endif	// SYNFIG_DEBUG_LAYERS
+		// resize the surface to the given render description
+		surface->set_wh(renddesc.get_w(),renddesc.get_h());
+		// and clear the surface
+		surface->clear();
+#ifdef SYNFIG_PROFILE_LAYERS
+		profile_timer.reset();
+#endif	// SYNFIG_PROFILE_LAYERS
+		return true;
+	}
+	
+#ifdef SYNFIG_DEBUG_LAYERS
+	synfig::info("Context::accelerated_render(): Descending into %s",(*context)->get_name().c_str());
+#endif	// SYNFIG_DEBUG_LAYERS
+	
+	try {
+		// lock the context for reading
+		RWLock::ReaderLock lock((*context)->get_rw_lock());
+#ifdef SYNFIG_PROFILE_LAYERS
+		//go down one layer :P
+		depth++;
+		curr_layer=(*context)->get_name();	//make sure the layer inside is referring to the correct layer outside
+		profile_timer.reset(); 										// +
+#endif	// SYNFIG_PROFILE_LAYERS
+		bool ret;
+		// this layer doesn't draw anything onto the canvas we're
+		// rendering, but it uses straight blending, so we need to render
+		// the stuff under us and then blit transparent pixels over it
+		// using the appropriate 'amount'
+		if (straight_and_empty)
+		{
+			if ((ret = Context((context+1)).accelerated_render(surface,quality,renddesc,cb)))
+			{
+				Surface clearsurface;
+				clearsurface.set_wh(renddesc.get_w(),renddesc.get_h());
+				clearsurface.clear();
+				Surface::alpha_pen apen(surface->begin());
+				apen.set_alpha(composite->get_amount());
+				apen.set_blend_method(composite->get_blend_method());
+				
+				clearsurface.blit_to(apen);
+			}
+		}
+		else
+			ret = (*context)->accelerated_render(context+1,surface,quality,renddesc, cb);
+#ifdef SYNFIG_PROFILE_LAYERS
+		//post work for the previous layer
+		time_table[curr_layer]+=profile_timer();							//-
+		if(run_table.count(curr_layer))run_table[curr_layer]++;
+		else run_table[curr_layer]=1;
+		depth--;
+		curr_layer = layer_name; //we are now onto this layer (make sure the post gets recorded correctly...
+		//print out the table it we're done...
+		if(depth==0) _print_profile_report(),time_table.clear(),run_table.clear();
+		profile_timer.reset();												//+
+#endif	// SYNFIG_PROFILE_LAYERS
+		return ret;
+	}
+	catch(std::bad_alloc)
+	{
+		synfig::error("Context::accelerated_render(): Layer \"%s\" threw a bad_alloc exception!",(*context)->get_name().c_str());
+#ifdef _DEBUG
+		return false;
+#else  // _DEBUG
+		++context;
+		return context.accelerated_render(surface, quality, renddesc, cb);
+#endif	// _DEBUG
+	}
+	catch(...)
+	{
+		synfig::error("Context::accelerated_render(): Layer \"%s\" threw an exception, rethrowing...",(*context)->get_name().c_str());
+		throw;
+	}
+}
+
+
+bool
+Context::accelerated_cairorender(cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb) const
+{
+#ifdef SYNFIG_PROFILE_LAYERS
+	String layer_name(curr_layer);
+	//sum the pre-work done by layer above us... (curr_layer is layer above us...)
+	if(depth>0)
+	{
+		time_table[curr_layer]+=profile_timer();
+	}
+#endif	// SYNFIG_PROFILE_LAYERS
+	
+	const Rect bbox(renddesc.get_rect());
+	// this is going to be set to true if this layer contributes
+	// nothing, but it's a straight blend with non-zero amount, and so
+	// it has an effect anyway
+	bool straight_and_empty = false;
+	etl::handle<Layer_Composite> composite;
+	Context context(*this);
+	// Run all layers until context is empty
+	for(;!(context)->empty();++context)
+	{
+		// If we are not active then move on to next layer
+		if(!(*context)->active())
+			continue;
+		const Rect layer_bounds((*context)->get_bounding_rect());
+		// Cast current layer to composite
+		composite = etl::handle<Layer_Composite>::cast_dynamic(*context);
+		// If the box area is less than zero or the boxes do not
+		// intersect then move on to next layer, unless the layer is
+		// using a straight blend and has a non-zero amount, in which
+		// case it will still affect the result
+		if(layer_bounds.area() <= 0.0000000000001 || !(layer_bounds && bbox))
+		{
+			if (composite &&
+				CairoSurface::value_type::is_straight(composite->get_blend_method()) &&
+				composite->get_amount() != 0.0f)
+			{
+				straight_and_empty = true;
+				break;
+			}
+			continue;
+		}
+		// If this layer has Straight as the blend method and amount
+		// is 1.0, and the layer doesn't depend on its context, then
+		// we don't want to render the context
+		if (composite &&
+			composite->get_blend_method() == Color::BLEND_STRAIGHT &&
+			composite->get_amount() == 1.0f &&
+			!composite->reads_context())
+		{
+			Layer::Handle layer = *context;
+			while (!context->empty()) context++; // skip the context
+			return layer->accelerated_cairorender(context,surface,quality,renddesc, cb);
+		}
+		// Break out of the loop--we have found a good layer
+		break;
+	}
+	// If this layer isn't defined, return alpha
+	if (context->empty() || (straight_and_empty && composite->get_amount() == 1.0f))
+	{
+#ifdef SYNFIG_DEBUG_LAYERS
+		synfig::info("Context::accelerated_render(): Hit end of list");
+#endif	// SYNFIG_DEBUG_LAYERS
+		// clear the surface
+		cairo_t* cr=cairo_create(surface);
+		cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
+		cairo_set_source_rgba(cr, 0, 0, 0, 0); // TODO: this line may be unnecessary
+		cairo_paint(cr);
+		cairo_destroy(cr);
+#ifdef SYNFIG_PROFILE_LAYERS
+		profile_timer.reset();
+#endif	// SYNFIG_PROFILE_LAYERS
+		return true;
+	}
+	
+#ifdef SYNFIG_DEBUG_LAYERS
+	synfig::info("Context::accelerated_render(): Descending into %s",(*context)->get_name().c_str());
+#endif	// SYNFIG_DEBUG_LAYERS
+	
+	try {
+		// lock the context for reading
+		RWLock::ReaderLock lock((*context)->get_rw_lock());
+#ifdef SYNFIG_PROFILE_LAYERS
+		//go down one layer :P
+		depth++;
+		curr_layer=(*context)->get_name();	//make sure the layer inside is referring to the correct layer outside
+		profile_timer.reset(); 										// +
+#endif	// SYNFIG_PROFILE_LAYERS
+		bool ret;
+		// this layer doesn't draw anything onto the canvas we're
+		// rendering, but it uses straight blending, so we need to render
+		// the stuff under us and then blit transparent pixels over it
+		// using the appropriate 'amount'
+		if (straight_and_empty)
+		{
+			if ((ret = Context((context+1)).accelerated_cairorender(surface,quality,renddesc,cb)))
+			{
+				cairo_t* cr=cairo_create(surface);
+				cairo_set_source_rgba(cr, 0, 0, 0, 1.0-composite->get_amount()); // TODO: handle amount values outside of the range 0.0-1.0
+				cairo_set_operator(cr, CAIRO_OPERATOR_DEST_IN);
+				cairo_paint(cr);
+				cairo_destroy(cr);
+			}
+		}
+		else
+			ret = (*context)->accelerated_cairorender(context+1,surface,quality,renddesc, cb);
+#ifdef SYNFIG_PROFILE_LAYERS
+		//post work for the previous layer
+		time_table[curr_layer]+=profile_timer();							//-
+		if(run_table.count(curr_layer))run_table[curr_layer]++;
+		else run_table[curr_layer]=1;
+		depth--;
+		curr_layer = layer_name; //we are now onto this layer (make sure the post gets recorded correctly...
+		//print out the table it we're done...
+		if(depth==0) _print_profile_report(),time_table.clear(),run_table.clear();
+		profile_timer.reset();												//+
+#endif	// SYNFIG_PROFILE_LAYERS
+		return ret;
+	}
+	catch(std::bad_alloc)
+	{
+		synfig::error("Context::accelerated_render(): Layer \"%s\" threw a bad_alloc exception!",(*context)->get_name().c_str());
+#ifdef _DEBUG
+		return false;
+#else  // _DEBUG
+		++context;
+		return context.accelerated_cairorender(surface, quality, renddesc, cb);
+#endif	// _DEBUG
+	}
+	catch(...)
+	{
+		synfig::error("Context::accelerated_render(): Layer \"%s\" threw an exception, rethrowing...",(*context)->get_name().c_str());
+		throw;
+	}
+}
