@@ -7,7 +7,7 @@
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2008 Chris Moore
-**  Copyright (c) 2011 Carlos López
+**  Copyright (c) 2011-2013 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -39,6 +39,10 @@
 #include "node.h"
 #include "time.h"
 #include "guid.h"
+#include "target.h" // for RenderMethod. TODO: put RenderMethod apart
+
+#include "cairo.h"
+#include "rendermethod.h"
 
 /* === M A C R O S ========================================================= */
 
@@ -160,10 +164,12 @@ class ValueNode;
 class ValueBase;
 class Time;
 class Surface;
+class CairoSurface;
 class RendDesc;
 class ProgressCallback;
 class Context;
 class Color;
+class CairoColor;
 class Transform;
 class Rect;
 class GUID;
@@ -502,7 +508,11 @@ public:
 	**	\see Context::get_color()
 	*/
 	virtual Color get_color(Context context, const Point &pos)const;
+	virtual CairoColor get_cairocolor(Context context, const Point &pos)const;
 
+	//! Sets the render method to the layer. Not all layers uses this virtual member
+	virtual void set_render_method(Context context, RenderMethod x);
+	
 	//! Renders the Canvas to the given Surface in an accelerated manner
 	/*!	\param context		Context iterator referring to next Layer.
 	**	\param surface		Pointer to Surface to render to.
@@ -513,6 +523,7 @@ public:
 	**	\see Context::accelerated_render()
 	*/
 	virtual bool accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
+	virtual bool accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
 
 	//! Checks to see if a part of the layer is directly under \a point
 	/*!	\param context		Context iterator referring to next Layer.
