@@ -7,7 +7,7 @@
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2008 Chris Moore
-**	Copyright (c) 2011 Carlos López
+**	Copyright (c) 2011-2013 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -121,7 +121,7 @@ Layer_Composite::accelerated_render(Context context,Surface *surface,int quality
 	image.push_front(const_cast<synfig::Layer_Composite*>(this));
 
 	// Set up a surface target
-	Target::Handle target(surface_target(surface));
+	Target_Scanline::Handle target(surface_target(surface));
 
 	if(!target)
 	{
@@ -138,6 +138,66 @@ Layer_Composite::accelerated_render(Context context,Surface *surface,int quality
 	//return render_threaded(Context(image.begin()),target,desc,&stagetwo,2);
 }
 
+/////
+
+bool
+Layer_Composite::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc_, ProgressCallback *cb)  const
+{
+	RendDesc renddesc(renddesc_);
+	
+	if(!amount)
+		return context.accelerated_cairorender(surface,quality,renddesc,cb);
+	
+	CanvasBase image;
+	
+	SuperCallback stageone(cb,0,50000,100000);
+	SuperCallback stagetwo(cb,50000,100000,100000);
+	
+	Layer_Bitmap::Handle surfacelayer(new class Layer_Bitmap());
+	
+	Context iter;
+	
+	for(iter=context;*iter;iter++)
+		image.push_back(*iter);
+	
+	// Add one Bitmap Layer on top
+	image.push_front(surfacelayer.get());
+		
+	image.push_back(0);	// and Alpha black at end
+	
+	// Render the backdrop on the surface layer's surface.
+	cairo_surface_t* cs=cairo_image_surface_create(CAIRO_FORMAT_ARGB32, renddesc.get_w(), renddesc.get_h());
+	if(!context.accelerated_cairorender(cs,quality,renddesc,&stageone))
+	{
+		cairo_surface_destroy(cs);
+		return false;
+	}
+	surfacelayer->set_cairo_surface(cs);
+	cairo_surface_destroy(cs);
+	// Sets up the interpolation of the context (now the surface layer is the first one)
+	// depending on the quality
+	if(quality<=4)surfacelayer->c=3;else
+		if(quality<=5)surfacelayer->c=2;
+		else if(quality<=6)surfacelayer->c=1;
+		else surfacelayer->c=0;
+	surfacelayer->tl=renddesc.get_tl();
+	surfacelayer->br=renddesc.get_br();
+	// Sets the blend method to straight. See below
+	surfacelayer->set_blend_method(Color::BLEND_STRAIGHT);
+	surfacelayer->set_render_method(context, CAIRO);
+	// Push this layer on the image. The blending result is only this layer
+	// and the surface layer. The rest of the context is ignored by the straight
+	// blend method of surface layer
+	image.push_front(const_cast<synfig::Layer_Composite*>(this));		
+
+	// Render the scene
+	return cairorender(Context(image.begin()),surface,renddesc,&stagetwo);
+}
+
+
+
+/////
+
 Rect
 Layer_Composite::get_full_bounding_rect(Context context)const
 {
