@@ -30,6 +30,7 @@
 
 #include "layer.h"
 #include "color.h"
+#include "cairo_operators.h"
 
 /* === M A C R O S ========================================================= */
 
@@ -89,6 +90,7 @@ public:
 	virtual Rect get_full_bounding_rect(Context context)const;
 	//! Renders the layer composited on the context and puts it on the target surface.
 	virtual bool accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
+	virtual bool accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
 }; // END of class Layer_Composite
 
 }; // END of namespace synfig
