@@ -7,7 +7,7 @@
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2007, 2008 Chris Moore
-**	Copyright (c) 2011 Carlos López
+**	Copyright (c) 2011-2013 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -75,7 +75,7 @@ public:
 
 SYNFIG_LAYER_INIT(Layer_PasteCanvas);
 SYNFIG_LAYER_SET_NAME(Layer_PasteCanvas,"PasteCanvas"); // todo: use paste_canvas
-SYNFIG_LAYER_SET_LOCAL_NAME(Layer_PasteCanvas,N_("Paste Canvas"));
+SYNFIG_LAYER_SET_LOCAL_NAME(Layer_PasteCanvas,N_("Group"));
 SYNFIG_LAYER_SET_CATEGORY(Layer_PasteCanvas,N_("Other"));
 SYNFIG_LAYER_SET_VERSION(Layer_PasteCanvas,"0.1");
 SYNFIG_LAYER_SET_CVS_ID(Layer_PasteCanvas,"$Id$");
@@ -117,8 +117,8 @@ Layer_PasteCanvas::~Layer_PasteCanvas()
 String
 Layer_PasteCanvas::get_local_name()const
 {
-	if(!canvas)	return _("Pasted Canvas");
-	if(canvas->is_inline()) return _("Inline Canvas");
+	if(!canvas)	return _("Group");
+	if(canvas->is_inline()) return _("Group");
 	if(canvas->get_root()==get_canvas()->get_root()) return '[' + canvas->get_id() + ']';
 
 	return '[' + canvas->get_file_name() + ']';
@@ -135,7 +135,7 @@ Layer_PasteCanvas::get_param_vocab()const
 	);
 	ret.push_back(ParamDesc("canvas")
 		.set_local_name(_("Canvas"))
-		.set_description(_("Canvas to paste"))
+		.set_description(_("Group content"))
 	);
 	ret.push_back(ParamDesc("zoom")
 		.set_local_name(_("Zoom"))
@@ -561,6 +561,137 @@ Layer_PasteCanvas::accelerated_render(Context context,Surface *surface,int quali
 	return true;
 }
 
+///////
+
+bool
+Layer_PasteCanvas::accelerated_cairorender(Context context,cairo_surface_t *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
+{
+	if(cb && !cb->amount_complete(0,10000)) return false;
+	
+	if(depth==MAX_DEPTH)
+		// if we are at the extent of our depth,
+		// then we should just return whatever is under us.
+		return context.accelerated_cairorender(surface,quality,renddesc,cb);
+	
+	depth_counter counter(depth);
+	
+	if(!canvas || !get_amount())
+		return context.accelerated_cairorender(surface,quality,renddesc,cb);
+	
+	SuperCallback stageone(cb,0,4500,10000);
+	SuperCallback stagetwo(cb,4500,9000,10000);
+	SuperCallback stagethree(cb,9000,9999,10000);
+	
+	RendDesc desc(renddesc);
+	Vector::value_type zoomfactor=1.0/exp(zoom);
+	desc.clear_flags();
+	desc.set_tl((desc.get_tl()-focus-origin)*zoomfactor+focus);
+	desc.set_br((desc.get_br()-focus-origin)*zoomfactor+focus);
+	desc.set_flags(RendDesc::PX_ASPECT);
+	
+	if (is_solid_color() || context->empty())
+	{
+		// clear the surface
+		cairo_t* cr=cairo_create(surface);
+		cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
+		cairo_paint(cr);
+		cairo_destroy(cr);
+	}
+	else if (!context.accelerated_cairorender(surface,quality,renddesc,&stageone))
+		return false;
+	
+	Real grow_value(get_parent_canvas_grow_value());
+	canvas->set_grow_value(outline_grow+grow_value);
+	
+	if(muck_with_time_ && curr_time!=Time::begin() /*&& canvas->get_time()!=curr_time+time_offset*/)
+		canvas->set_time(curr_time+time_offset);
+	
+	Color::BlendMethod blend_method(get_blend_method());
+	const Rect full_bounding_rect(canvas->get_context().get_full_bounding_rect());
+	
+	bool blend_using_straight = false; // use 'straight' just for the central blit
+	
+	// sometimes the user changes the parameters while we're
+	// rendering, causing our pasted canvas' bounding box to shrink
+	// and no longer overlap with our tile.  if that has happened,
+	// let's just stop now - we'll be refreshing soon anyway
+	//! \todo shouldn't a mutex ensure this isn't needed?
+	// http://synfig.org/images/d/d2/Bbox-change.sifz is an example
+	// that shows this happening - open the encapsulation, select the
+	// 'shade', and toggle the 'invert' parameter quickly.
+	// Occasionally you'll see:
+	//   error: Context::accelerated_render(): Layer "shade" threw a bad_alloc exception!
+	// where the shade layer tries to allocate itself a canvas of
+	// negative proportions, due to changing bounding boxes.
+	if (!etl::intersect(desc.get_rect(), full_bounding_rect))
+	{
+		warning("%s:%d bounding box shrank while rendering?", __FILE__, __LINE__);
+		return true;
+	}
+	
+	// we have rendered what's under us, if necessary
+	if(context->empty())
+	{
+		// if there's nothing under us, and we're blending 'onto', then we've finished
+		if (Color::is_onto(blend_method)) return true;
+		
+		// there's nothing under us, so using straight blending is
+		// faster than and equivalent to using composite, but we don't
+		// want to blank the surrounding areas.
+		if (blend_method==Color::BLEND_COMPOSITE) blend_using_straight = true;
+	}
+
+	Rect area(desc.get_rect() & full_bounding_rect);
+	
+	Point min(area.get_min());
+	Point max(area.get_max());
+	
+	if (desc.get_tl()[0] > desc.get_br()[0]) swap(min[0], max[0]);
+	if (desc.get_tl()[1] > desc.get_br()[1]) swap(min[1], max[1]);
+	
+	const int x(floor_to_int((min[0] - desc.get_tl()[0]) / desc.get_pw()));
+	const int y(floor_to_int((min[1] - desc.get_tl()[1]) / desc.get_ph()));
+	const int w( ceil_to_int((max[0] - desc.get_tl()[0]) / desc.get_pw()) - x);
+	const int h( ceil_to_int((max[1] - desc.get_tl()[1]) / desc.get_ph()) - y);
+		
+	desc.set_subwindow(x,y,w,h);
+	
+	if(desc.get_w()==0 || desc.get_h()==0)
+	{
+		if(cb && !cb->amount_complete(10000,10000)) return false;
+		return true;
+	}
+
+	if (!etl::intersect(context.get_full_bounding_rect(),(full_bounding_rect-focus)*exp(zoom)+origin+focus))
+	{
+		// if there's no intersection between the context and our
+		// surface, and we're rendering 'onto', then we're done
+		if (Color::is_onto(blend_method) && !Color::is_straight(blend_method))
+			return true;
+	}
+	
+	// render the canvas to be pasted onto pastesurface
+	cairo_surface_t* pastesurface=cairo_surface_create_similar_image(surface, CAIRO_FORMAT_ARGB32, desc.get_w(), desc.get_h());
+	if(!canvas->get_context().accelerated_cairorender(pastesurface,quality,desc,&stagetwo))
+		return false;
+
+	cairo_t *cr = cairo_create(surface);
+	cairo_set_source_surface(cr, pastesurface, x, y);
+	cairo_surface_destroy(pastesurface);
+	cairo_paint_with_alpha_operator(cr, get_amount(), blend_using_straight ? Color::BLEND_STRAIGHT : blend_method);
+	cairo_destroy(cr);
+	
+	if(cb && !cb->amount_complete(10000,10000)) return false;
+	
+	return true;
+}
+
+
+///////
+
+
+
+
 Rect
 Layer_PasteCanvas::get_bounding_rect()const
 {
@@ -601,3 +732,13 @@ Layer_PasteCanvas::get_param_static(const String &param) const
 	return Layer_Composite::get_param_static(param);
 }
 
+void
+Layer_PasteCanvas::set_render_method(Context context, RenderMethod x)
+{
+	if(canvas) // if there is a canvas pass down to it
+		canvas->get_context().set_render_method(x);
+
+	// in any case pass it down
+	context.set_render_method(x);
+}
+
