@@ -9,7 +9,8 @@
 **	Copyright (c) 2007, 2008 Chris Moore
 **	Copyright (c) 2009 Carlos A. Sosa Navarro
 **	Copyright (c) 2009 Nikita Kitaev
-**  Copyright (c) 2011, 2012 Carlos López
+**	Copyright (c) 2011, 2012 Carlos López
+**	Copyright (c) 2012-2013 Konstantin Dmitriev
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -212,7 +213,7 @@ CanvasParser::error(xmlpp::Node *element, const String &text)
 	errors_text += "  * " + str + "\n";
 	if(!allow_errors_)
 		throw runtime_error(str);
-	cerr<<str<<endl;
+	cerr<<str.c_str()<<endl;
 	//	synfig::error(str);
 }
 
@@ -239,13 +240,19 @@ CanvasParser::parse_keyframe(xmlpp::Element *element,Canvas::Handle canvas)
 	Keyframe ret(Time(element->get_attribute("time")->get_value(),canvas->rend_desc().get_frame_rate()));
 
 
-	if(element->get_children().empty())
-		return ret;
-
-	if(element->get_child_text()->get_content().empty())
-		return ret;
-
-	ret.set_description(element->get_child_text()->get_content());
+	if(!element->get_children().empty())
+		if(!element->get_child_text()->get_content().empty())
+			ret.set_description(element->get_child_text()->get_content());
+	
+	bool active=true;
+	if(element->get_attribute("active")) 
+	{
+		string val=element->get_attribute("active")->get_value();
+		if(val=="false" || val=="0")
+			active=false;
+	}
+	ret.set_active(active);
+		
 
 	return ret;
 }
@@ -2460,7 +2467,7 @@ CanvasParser::parse_canvas(xmlpp::Element *element,Canvas::Handle parent,bool in
 			if(child->get_name()=="defs")
 			{
 				if(canvas->is_inline())
-					error(child,_("Inline canvas cannot have a <defs> section"));
+					error(child,_("Group canvases cannot have a <defs> section"));
 				parse_canvas_defs(child, canvas);
 			}
 			else
@@ -2468,7 +2475,7 @@ CanvasParser::parse_canvas(xmlpp::Element *element,Canvas::Handle parent,bool in
 			{
 				if(canvas->is_inline())
 				{
-					warning(child,_("Inline canvas cannot have keyframes"));
+					warning(child,_("Group canvases cannot have keyframes"));
 					continue;
 				}
 
@@ -2480,7 +2487,7 @@ CanvasParser::parse_canvas(xmlpp::Element *element,Canvas::Handle parent,bool in
 			{
 				if(canvas->is_inline())
 				{
-					warning(child,_("Inline canvases cannot have metadata"));
+					warning(child,_("Group canvases cannot have metadata"));
 					continue;
 				}
 
@@ -2661,7 +2668,7 @@ CanvasParser::parse_from_file_as(const String &file_,const String &as_,String &e
 	}
 	catch(const String& str)
 	{
-		cerr<<str<<endl;
+		cerr<<str.c_str()<<endl;
 		//	synfig::error(str);
 		errors = str;
 		return Canvas::Handle();
@@ -2717,7 +2724,7 @@ CanvasParser::parse_as(xmlpp::Element* node,String &errors)
 	}
 	catch(const String& str)
 	{
-		cerr<<str<<endl;
+		cerr<<str.c_str()<<endl;
 		errors = str;
 		return Canvas::Handle();
 	}
