@@ -518,6 +518,31 @@ RendDesc::set_br(const Point &x)
 	br_=x; return *this;
 }
 
+RendDesc &
+RendDesc::set_tl_br(const Point &x, const Point &y)
+{
+	if(FLAGS(flags, PX_ASPECT))
+	{
+		Vector new_size(y-x);
+		new_size[0]=abs(new_size[0]);
+		new_size[1]=abs(new_size[1]);
+		
+		Vector old_size(tl_-br_);
+		old_size[0]=abs(old_size[0]);
+		old_size[1]=abs(old_size[1]);
+		
+		if(new_size[0]!=old_size[0])
+			w_=round_to_int(new_size[0]*w_/old_size[0]);
+		
+		if(new_size[1]!=old_size[1])
+			h_=round_to_int(new_size[1]*h_/old_size[1]);
+	}
+	tl_=x;
+	br_=y;
+	return *this;
+}
+
+
 RendDesc &
 RendDesc::set_viewport(const Point &__tl, const Point &__br)
 { tl_=__tl; br_=__br; return *this; }
@@ -555,3 +580,25 @@ RendDesc::set_subwindow(int x, int y, int w, int h)
 
 	return *this;
 }
+
+RendDesc &
+RendDesc::set_duration(Time duration)
+{
+	if(get_frame_rate())
+	{
+		if(duration > Time(0.0))
+			set_time_end(get_time_start() + duration  - Time(1/get_frame_rate()));
+		else
+			set_time_end(get_time_start());
+	}
+		
+	return *this;
+}
+
+const Time
+RendDesc::get_duration()
+{
+	if(get_frame_rate())		
+		return (get_time_end() - get_time_start() + Time(1/get_frame_rate()));
+	return Time(0.0);
+}
