@@ -281,6 +281,10 @@ public:
 	const Point &get_br()const;
 	//! Sets the bottom right point of the compostion
 	RendDesc &set_br(const Point &x);
+	//! Sets the top left and the bottom right of the composition
+	// Use this when the individual set_tl or set_br
+	// produce degenerate w or h
+	RendDesc &set_tl_br( const Point &x, const Point &y);
 	//! Returns the rectangle of the composition
 	Rect get_rect()const { return Rect(get_tl(),get_br()); }
 	//! Sets the view port by the top left and right bottom corners
@@ -293,6 +297,11 @@ public:
 	Real get_ph()const;
 	//! Sets viewport to represent the screen at the given pixel coordinates
 	RendDesc &set_subwindow(int x, int y, int w, int h);
+	//! Sets the duration of the animation. 
+	// Keeps the start time and modifies the end time to match the duration
+	RendDesc &set_duration(Time t);
+	//! Gets the duration of the animation
+	const Time get_duration();
 };	// END of class RendDesc
 
 //! This operator allows the combining of RendDesc::Lock flags using the '|' operator
