@@ -6,6 +6,7 @@
 **
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
+**  Copyright (c) 2011 Carlos López
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -40,6 +41,7 @@ class ValueNode_BLineCalcVertex : public LinkableValueNode
 	ValueNode::RHandle bline_;
 	ValueNode::RHandle loop_;
 	ValueNode::RHandle amount_;
+	ValueNode::RHandle homogeneous_;
 
 	ValueNode_BLineCalcVertex(const ValueBase::Type &x=ValueBase::TYPE_VECTOR);
 
@@ -56,11 +58,6 @@ public:
 	virtual String get_local_name()const;
 
 	virtual ValueNode::LooseHandle get_link_vfunc(int i)const;
-	virtual int link_count()const;
-	virtual String link_name(int i)const;
-
-	virtual String link_local_name(int i)const;
-	virtual int get_link_index_from_name(const String &name)const;
 
 protected:
 	LinkableValueNode* create_new()const;
@@ -71,6 +68,7 @@ public:
 	using synfig::LinkableValueNode::set_link_vfunc;
 	static bool check_type(ValueBase::Type type);
 	static ValueNode_BLineCalcVertex* create(const ValueBase &x=ValueBase::TYPE_VECTOR);
+	virtual Vocab get_children_vocab_vfunc()const;
 }; // END of class ValueNode_BLineCalcVertex
 
 }; // END of namespace synfig
